# Platform Video <script type="text/javascript">var pagestartms = Date.now();</script>

[![video-head](./support/video-head.png)](http://www.foxsports.com/watch/garbage-time-with-katie-nolan?vid=413455939774)

An overview of collected data used to identify issues affecting [video performance][5]. [Download this application][12] as a package and run locally.

Credit **Justin Reade**, **Hector Reyes**, **Chris DePauw**.

-------------------------------------------------------------------------------
# Setup

Define video and advertisement values, then 'play' or visit urls where same content is seen in foxsports.com. Similar to the [neodigital reference player][1], but with added control to define interactive unit and video id.

<div class="video-content">
   <div id="video-player"></div>
</div><div id="video control" class="video-control">
 <form name="neoform">
  <label for="videosubmit">
    <input type="submit" id="videosubmit" name="videosubmit" value="play" />
  </label>
  <label for="videoclear">
    <button type="button" id="videoclear" name="videoclear">clear</button>
  </label>
  <fieldset>
   <legend>NEO params</legend>
   <label for="version">
    <div>version</div>
    <select name="version" id="version">
      <option value="current" select="selected">current</option>
      <option value="prod">prod</option>
      <option value="dev">dev</option>
      <option value="qa">qa</option>
    </select>
   </label>
   <label for="autoplay">
    <div>autoplay</div>
    <select name="autoplay" id="autoplay">
      <option value="true" select="selected">true</option>
      <option value="false">false</option>
    </select>
   </label>
  </fieldset><fieldset>
   <legend>Release URL</legend>
   <label for="videoid">
    <div>videoid</div>
    <select name="videoid" id="videoid">
      <option value="Sdv8DTyUPsp_" select="selected">Sdv8DTyUPsp\_</option>
      <option value="UEa9ldtRlgx_">UEa9ldtRlgx\_</option>
    </select>
   </label>
   <label for="iu">
    <div>iu (interactive unit)</div>
    <select name="iu" id="iu">
      <option select="selected" value="/fscom/homepage/video" select="selected">/fscom/homepage/video</option>
      <option value="/fscom/nfl/video" select="selected">/fscom/nfl/video</option>
      <option value="" select="selected">no ad</option>
    </select>
   </label>
  </fieldset>
  <ul>
    <li>new tab qa4.foxsports.com **see console for results**
       <a target="_blank" id="testservicelink" href="#">
       </a></li><li>new tab qa4.foxsports.com using _sequential strategy_ **see console for results**
       <a target="_blank" id="testserviceqalink" href="#">
       </a></li><li>new tab this page anew, (_control strategy_) **see console for results**
       <a target="_blank" id="localservicelink" href="#">
       </a></li><li>new tab neo digital reference player
       <a target="_blank" id="referenceservicelink" href="#"></a>
    </li>
 </ul>
 </form>
</div>


-------------------------------------------------------------------------------
# How

Scripting added to the application is used to measure important video time sequences. 'Yields a consistent data for various form factors, environments, browsers, etc...

*Added to document head*:
```html
<head>
  <script type="text/javascript">
    var pagestartms = Date.now();
  </script>
```

*Added at bottom of document, above our app.js script element*:
```html
  <script type="text/javascript">
    var scriptstartms = Date.now();
  </script>
  <script src="${path:getEx...app.full.js...</script>
```

*Added to occur at 'platformPlayer' init method*:
```javascript
var videoinitms = Date.now();
```

*Added to occur at 'platformPlayer' init completion where video play starts*:
```javascript
var videostartms = Date.now();
```

Data is [printed to console in JSON format][11], for example:
```json
{
  "browserName" : "Chrome",
  "browserFullVersion" : "41.0.2272.89",
  "browserMajorVersion" : "41",
  "navigatorappName" : "Netscape",
  "navigatoruserAgent" : "Mozilla/5.0 (Maci ... ",
  "whArr" : [1750, 376],
  "fullURL" : "http%3A%2F%2Fchris.foxsports ... ",
  "releaseURL" : "http%3A%2F%2Flink.theplat ... ",
  "isCached" : true,
  "strategy" : "current",
  "err" : "null",
  "page-load to video-start" : 33.34,
    "page-load to video-init"   : 26.57,
      "page-load to script-start"  : 25.64,
      "script-start to video-init" : 0.93,
    "video-init to video-start" : 6.77
}
```

-------------------------------------------------------------------------------
# Experiment

We scripted a new initialization strategy and tested it alongside the current strategy.

 1. **Current strategy** initializes many definitions simultanaeously. Before platformPlayer is 'done', stacks generated by GlobalNavigation, Analytics and others appear on the event loop in uncontrolled ways.

    ```javascript
    viewfoxbox.init();
    Social.init();
    ModuleLoader.init();
      // ModuleLoader will 'init' more modules.
      // loops an array of modules calling 'init' on each.
    adManager.init();
    platformPlayer.init();
    Analytitics.init();
    Transitions.init();
    autocomplete.init();
    GlobalNavigqtion.init();
    InjectorScriptRunner.init();
    ```
    
    ![video-head](./support/screenshot-strategy-current.png)
    
 2. **Sequential strategy** initializes definitions in sequence. Until platformPlayer is 'done' (or expires) no other initialization begins.

    ```javascript
    var startms = Date.now();
    platformPlayer.init(opts, function (err, res) {
      if (err) {
        console.warn('error loading player');
      } else {
        console.log('finish loading player', Date.now() - startms);
      }
      loadnext();
    }, 1000); // expire, 1 second
    ```
    
    ![video-head](./support/screenshot-strategy-sequential.png)

 3. **Local strategy** initializes a the video to this page, which doesn't have much else.

 4. **¿Auto strategy?** use the [packaged version][12] of this application to run many automated tests using a headless browser.


-------------------------------------------------------------------------------
# Results

Filters below are applied to [this collection][11] of JSON data.

<form name="resultsform">
 <fieldset>
   <legend>Results Filter</legend>
   <label for="browsername">
    <div>browser name</div>
    <select name="browsername" id="browsername">
      <option value="" select="selected">---</option>
    </select>
   </label>
   <label for="releaseurl">
    <div>releaseURL</div>
    <select name="releaseurl" id="releaseurl">
      <option value="" select="selected">---</option>
    </select>
   </label>
   <label for="fullurl">
    <div>fullURL</div>
    <select name="fullurl" id="fullurl">
      <option value="" select="selected">---</option>
    </select>
   </label>
   <label for="iscache">
    <div>cache</div>
    <select name="iscache" id="iscache">
      <option value="" select="selected">---</option>
    </select>
   </label>
 </fieldset> 
</form>

|                                | **Control** | **Current** | **Sequential** |
|:------------------------------ |:----------- |:----------- |:-------------- |
| page-start to video-start      | ---         | ---         | ---            |
| **page-start to video-init**   | **---**     | **---**     | **---**        |
|   page-start to script-start   |  ---        |  ---        | ---            |
|   script-start to video-init   |  ---        |  ---        | ---            |
| **video-init to video-start**  | **---**     | **---**     | **---**        |


<script type="text/javascript">var scriptstartms = Date.now();</script>
   


[1]: http://player.foxneodigital.com/v1.6.2_554/reference    "reference player"
[3]: http://player.foxneodigital.com/v1.6.2_554/reference/?network=sports&version=qa&debug=&elem=video-player&width=640&height=360&releaseURL=http%3A%2F%2Flink.theplatform.com%2Fs%2FBKQ29B%2FUEa9ldtRlgx_%3Fmbr%3Dtrue%26policy%3D51457&pre=&post=&autoplay=&deliveryMode=&golive_show=&liveCC=&introURL=&outroURL=&introLink=&outroLink=&waitTime=&share=&share_deeplink=&share_deeplinkfunc=&share_showembedfunc=&share_email=&share_emailscript=&share_embed=&share_fb=&share_omitseries=&hidePostup=&share_iframeurl=&endcard=true&endcard_feed=http%3A%2F%2Ffeed.theplatform.com%2Ff%2FBKQ29B%2Ffoxsports-all%2F%3F%26startIndex%3D2%26params%3Diu%3D%2Ffscom%2Ftest%2Fvideo-performance&endcard_playlist=&endcard_related=&endcard_countdown=&endcard_autoadvance=&watermark_show=&watermark_src=&watermark_corner=&watermark_opacity=&play_overlay_show=&play_overlay_x_offset=&play_overlay_y_offset=&siteSection=&tremorID=&freewheel_keyvalue=&submit=Submit
[4]: http://player.foxneodigital.com/v1.6.2_554/reference/?network=sports&version=current&debug=&elem=video-player&width=640&height=360&releaseURL=http%3A%2F%2Flink.theplatform.com%2Fs%2FBKQ29B%2FSdv8DTyUPsp_%3Fmbr%3Dtrue%26assetTypes%3DVideo%26feed%3DFox%2520Sports%2520All%26iu%3D%2Ffscom%2Fhomepage%2Fvideo&pre=&post=&autoplay=&deliveryMode=&golive_show=&liveCC=&introURL=&outroURL=&introLink=&outroLink=&waitTime=&share=&share_deeplink=&share_deeplinkfunc=&share_showembedfunc=&share_email=&share_emailscript=&share_embed=&share_fb=&share_omitseries=&hidePostup=&share_iframeurl=&endcard=&endcard_feed=&endcard_playlist=&endcard_related=&endcard_countdown=&endcard_autoadvance=&watermark_show=&watermark_src=&watermark_corner=&watermark_opacity=&play_overlay_show=&play_overlay_x_offset=&play_overlay_y_offset=&siteSection=&tremorID=&freewheel_keyvalue=&submit=Submit
[5]: https://foxnetworksgroup.atlassian.net/browse/FSCOM-3135      "JIRA - video performance"
[6]: ./support/videoresults.json                                                "JSON results"


[8]: http://qa4.foxsports.com/nfl/video?vid=418805827517&videotest=true&releaseURL=http%3A%2F%2Flink.theplatform.com%2Fs%2FBKQ29B%2FSdv8DTyUPsp_%3Fmbr%3Dtrue%26assetTypes%3DVideo%26feed%3DFox%2520Sports%2520All%26iu%3D%2Ffscom%2Fhomepage%2Fvideo
[9]: http://qa4.foxsports.com/nfl/video?vid=418805827517&videotest=true&load-alone=true&load-seq=true&releaseURL=http%3A%2F%2Flink.theplatform.com%2Fs%2FBKQ29B%2FSdv8DTyUPsp_%3Fmbr%3Dtrue%26assetTypes%3DVideo%26feed%3DFox%2520Sports%2520All%26iu%3D%2Ffscom%2Fhomepage%2Fvideo

[10]: http://www.foxsports.com/watch/garbage-time-with-katie-nolan?vid=413455939774 "Garbage Time"
[11]: ./support/results.json
[12]: ./support/videotesting.zip
